/*
 *  include/asm-arm/arm/arch-cpe/cpe_time.h
 *
 *  Timer definitions for A320 platform
 *
 * Copyright (C) 2005 Faraday Corp.
 *
 * ChangeLog:
 *   04/20/2005 Luke Lee add control flags.
 */


#ifndef _CPE_TIME_H_
#define _CPE_TIME_H_
#include <asm/arch/cpe/cpe.h>
#include <asm/arch/cpe_int.h>


#define TIMER1_COUNT                0x0
#define TIMER1_LOAD                 0x4
#define TIMER1_MATCH1               0x8
#define TIMER1_MATCH2               0xC
#define TIMER2_COUNT                0x10
#define TIMER2_LOAD                 0x14
#define TIMER2_MATCH1               0x18
#define TIMER2_MATCH2               0x1C
#define TIMER3_COUNT                0x20
#define TIMER3_LOAD                 0x24
#define TIMER3_MATCH1               0x28
#define TIMER3_MATCH2               0x2C 
#define TIMER_CR                    0x30
#define TIMER_INTRSTATE             0x34
#define TIMER_INTRMASK              0x38

#define TM1ENABLE                   1
#define TM1CLOCK                    (1<<1)
#define TM1OFENABLE                 (1<<2)
#define TM2ENABLE                   (1<<3)
#define TM2CLOCK                    (1<<4)
#define TM2OFENABLE                 (1<<5)
#define TM3ENABLE                   (1<<6)
#define TM3CLOCK                    (1<<7)
#define TM3OFENABLE                 (1<<8)
#define TM1UPDOWN                   (1<<9)
#define TM2UPDOWN                   (1<<10)
#define TM3UPDOWN                   (1<<11)

typedef struct
{       
	unsigned int Tm1En:1;					// Timer1 enable bit
	unsigned int Tm1Clock:1;				// Timer1 clock source (0: PCLK, 1: EXT1CLK)
	unsigned int Tm1OfEn:1;				// Timer1 over flow interrupt enable bit
	unsigned int Tm2En:1;
	unsigned int Tm2Clock:1;
	unsigned int Tm2OfEn:1;
	unsigned int Tm3En:1;
	unsigned int Tm3Clock:1;
	unsigned int Tm3OfEn:1;       
	unsigned int Tm1UpDown:1; 
	unsigned int Tm2UpDown:1; 
	unsigned int Tm3UpDown:1; 	      
	unsigned int Reserved;        
} cpe_time_ctrl_t;


typedef struct 
{
    unsigned int TimerValue;
    unsigned int TimerLoad;
    unsigned int TimerMatch1;
    unsigned int TimerMatch2;
} cpe_time_reg_t;

extern cpe_time_reg_t *TimerBase[];
extern unsigned int cpe_timer_enable(unsigned int timer);
extern unsigned int cpe_timer_disable(unsigned int timer);
extern void cpe_timer_set_counter(unsigned int timer, unsigned int value);
extern void cpe_timer_set_reload(unsigned int timer, unsigned int value);
extern unsigned int cpe_timer_get_counter(unsigned int timer);
extern struct irqaction timer_irq;
extern unsigned long cpe_gettimeoffset(void);

static inline void cpe_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
    do_timer(regs);
}

#if 0
static inline void setup_timer(void)
{
#ifdef TIMER_INC_MODE
    cpe_timer_set_reload(1, 0xffffffff - APB_CLK/HZ);
    cpe_timer_set_counter(1, 0xffffffff - APB_CLK/HZ);
#else
    cpe_timer_set_reload(1, APB_CLK/HZ);
    cpe_timer_set_counter(1, APB_CLK/HZ);
#endif

    if(!cpe_timer_enable(1))
    {
		panic("can not enable timer\n");
	}
	
	printk("IRQ timer at interrupt number 0x%x clock %d\r\n",IRQ_TIMER1,APB_CLK);
	cpe_int_set_irq(IRQ_TIMER1, EDGE,L_ACTIVE);
	timer_irq.handler = cpe_timer_interrupt;
	timer_irq.flags = SA_INTERRUPT; //ivan added
	setup_arm_irq(IRQ_TIMER1, &timer_irq);
}
#endif // 0

#endif
